unit iFont;

interface

uses
	Forms, Classes, Graphics;

(* ꎞItHgCXg[NX *)
type
	TInstallFont = class
	private
		FTrueTypeFontName: string;
		FInstalled	: Boolean;
    public
		constructor	Create(TTFName: string);
        destructor	Destroy; override;
		function	Install(): Boolean;
        procedure	UnInstall();
        function	ToString(): string;
        function	FontName(): string;
    protected
		//
    published
        property	FullName: string read FTrueTypeFontName;
    	property	Installed: Boolean read FInstalled;
    end;

type
	TSortType = (stName, stStatus);

(* ꎞItHgCXg[NXpXg *)
type
	TInstallFontList = class(TList)
    private
		FInfFile	: String;
        function	Get(Index: Integer): TInstallFont;
        procedure	Put(Index: Integer; const Value: TInstallFont);
    public
		function	Exists(TTFName: string): Boolean;
        property	Items[Index: Integer]: TInstallFont read Get write Put; default;
        procedure	Clear; override;
        procedure	Delete(Index: integer);
        constructor	Create(AInfFile: string);
        destructor	Destroy; override;
		procedure	SortEx(ASortType: TSortType = stName);
        procedure	SaveToFile;
        procedure	LoadFromFile;
        procedure	AddEx(AInstallFont: TInstallFont);
    end;

function	KanaToChars(AKana: string): string;

implementation

uses
	Windows, Messages, SysUtils, iUtils;

var
	KanaTable: TStrings;

(*
	=====================================================================
	TInstallFont
	=====================================================================
*)

(* RXgN^ *)
constructor	TInstallFont.Create(TTFName: string);
begin
	inherited Create;
    FTrueTypeFontName := TTFName;
    FInstalled := False;
end;

(* fXgN^ *)
destructor	TInstallFont.Destroy;
begin
	if FInstalled then UnInstall();
    inherited Destroy;
end;

(* tHgCXg[ *)
function	TInstallFont.Install(): Boolean;
begin
	if not FInstalled then
    begin
        //CXg[
        FInstalled := (AddFontResource(PChar(FTrueTypeFontName)) <> 0);
        //̃AvP[Vɂʒm
        if FInstalled then SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
    end;
	Result := FInstalled;
end;

(* tHgACXg[ *)
procedure	TInstallFont.UnInstall();
begin
	if FInstalled then
    begin
		//폜
        RemoveFontResource(PChar(FTrueTypeFontName));
		//̃AvP[Vɂʒm
        SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
        FInstalled := False;
    end;
end;

(* \ *)
function	TInstallFont.ToString(): string;
var
	Name: string;
begin
    Name := FontName();
	if FInstalled then
    	Result := ' ' + Name
    else
    	Result := ' ' + Name;
end;

(* tHg *)
function	TInstallFont.FontName(): string;
begin
	Result := ExtractFileName(FTrueTypeFontName);
end;

(*
	=======================================================
	TInstallFontListp\[g
	=======================================================
*)

(* t@C\[g *)
function	SortByName(Item1, Item2: Pointer): Integer;
var
	Temp1, Temp2: string;
begin
    Temp1 := ExtractFileName(TInstallFont(Item1).FTrueTypeFontName);
    Temp2 := ExtractFileName(TInstallFont(Item2).FTrueTypeFontName);
	if Temp1 > Temp2 then Result := 1 else
    if Temp1 = Temp2 then Result := 0 else
    Result := -1;
end;

(* CXg[󋵃\[g *)
function	SortByStatus(Item1, Item2: Pointer): Integer;
var
	Temp1, Temp2: Boolean;
begin
	Temp1 := TInstallFont(Item1).FInstalled;
	Temp2 := TInstallFont(Item1).FInstalled;
    if Temp1 <> Temp2 then
    begin
		if Temp1 then Result := 1 else Result := 0
    end
    else
    begin
		Result := SortByName(Item1, Item2);
    end;
end;

(*
	=======================================================
	TInstallFontList
	=======================================================
*)

(* tHg̑݃`FbN *)
function	TInstallFontList.Exists(TTFName: string): Boolean;
var
	i: integer;
    Name, CompName: string;
begin
	Result := True;
    CompName := UpperCase(ExtractFileName(TTFName));
	for i := 0 to Self.Count - 1 do
    begin
		Name := UpperCase(ExtractFileName(Self[i].FTrueTypeFontName));
        if CompName = Name then Exit;
    end;
	Result := False;
end;

(* Ƃ *)
function	TInstallFontList.Get(Index: Integer): TInstallFont;
begin
	Result := TInstallFont(inherited Get(Index));
end;

(* u *)
procedure	TInstallFontList.Put(Index: Integer; const Value: TInstallFont);
begin
	inherited Put(Index, Value);
end;

(* ACeNA *)
procedure	TInstallFontList.Clear;
begin
	while Self.Count > 0 do Self.Delete(0);
    inherited Clear;
end;

(* ACe폜 *)
procedure	TInstallFontList.Delete(Index: integer);
begin
    Self.Items[Index].Free;
    inherited Delete(Index);
end;

(* ACeǉEds *)
procedure	TInstallFontList.AddEx(AInstallFont: TInstallFont);
begin
	if not Self.Exists(AInstallFont.FullName) then
    begin
    	Self.Add(AInstallFont);
    end
    else
    begin
		AInstallFont.Free;
        AInstallFont := nil;
    end;
end;

(* RXgN^ *)
constructor	TInstallFontList.Create(AInfFile: string);
begin
	inherited Create;
    FInfFile := AInfFile;
end;

(* fXgN^ *)
destructor	TInstallFontList.Destroy;
begin
    Self.Clear;
	inherited Destroy;
end;

(* \[g *)
procedure	TInstallFontList.SortEx(ASortType: TSortType = stName);
begin
	case ASortType of
    	stName	: Self.Sort(@SortByName);
        stStatus: Self.Sort(@SortByStatus);
    end;
end;

(* ACeۑ *)
procedure	TInstallFontList.SaveToFile;
var
	i: integer;
    iStrings: TStrings; 
begin
    iStrings := TStringList.Create;
    try
    	for i := 0 to Self.Count - 1 do
        begin
            iStrings.Add(Self[i].FullName);
        end;
        iStrings.SaveToFile(FInfFile);
    finally
    	iStrings.Free;
    end;
end;

(* ACeǍ *)
procedure	TInstallFontList.LoadFromFile;
var
	i: integer;
    iStrings: TStrings; 
begin
	if FileExists(FInfFile) then
    begin
        iStrings := TStringList.Create;
        try
            iStrings.LoadFromFile(FInfFile);
            for i := 0 to iStrings.Count - 1 do
            begin
                try
                    if not IsFolderPath(iStrings[i]) then
                    begin
                        if CheckExtension(iStrings[i], '.ttf') then
                        begin
                        	Self.AddEx(TInstallFont.Create(iStrings[i]));
                        end;
                    end;
                except
					;
                end;
            end;
        finally
            iStrings.Free;
        end;
    end;
end;

(*
	=====================================================================
	TextMetrics
	=====================================================================
*)
{
procedure	iGetTextMetrics(ACanvas: TCanvas; AStrings: TStrings);
const
	VIEW_WIDTH = 26;
var
	FontInfo: TTextMetric;
    FaceName: array[0..255] of Char;
begin
	//tHg擾
	GetTextFace(ACanvas.Handle, SizeOf(FaceName), FaceName);
	//tHg̕擾
    GetTextMetrics(ACanvas.Handle, FontInfo);

    with AStrings, FontInfo do
    begin

    	Clear;
        Add(TitledString('FaceName:', FaceName, VIEW_WIDTH));

        //
        Add(TitledString('Height:', tmHeight, VIEW_WIDTH));
        //AZg
        Add(TitledString('Ascent:', tmAscent, VIEW_WIDTH));
        //fBZg
        Add(TitledString('Descent:', tmDescent, VIEW_WIDTH));
        //fBO
        Add(TitledString('InternalLeading:', tmInternalLeading, VIEW_WIDTH));
        //OfBO
        Add(TitledString('ExternalLeading:', tmExternalLeading, VIEW_WIDTH));
        //̕ϕ
        Add(TitledString('Average Char Width:', tmAveCharWidth, VIEW_WIDTH));
        //̍ő啝
        Add(TitledString('Max Char Width:', tmMaxCharWidth, VIEW_WIDTH));
        //{[hl
        case tmWeight of
            FW_DONTCARE:   Add(TitledString('Weight:', 'Don''t care', VIEW_WIDTH));
            FW_THIN:       Add(TitledString('Weight:', 'Thin', VIEW_WIDTH));
            FW_EXTRALIGHT: Add(TitledString('Weight:', 'Extra light', VIEW_WIDTH));
            FW_LIGHT:      Add(TitledString('Weight:', 'Light', VIEW_WIDTH));
            FW_NORMAL:     Add(TitledString('Weight:', 'Normal', VIEW_WIDTH));
            FW_MEDIUM:     Add(TitledString('Weight:', 'Medium', VIEW_WIDTH));
            FW_SEMIBOLD:   Add(TitledString('Weight:', 'Semibold', VIEW_WIDTH));
            FW_BOLD:       Add(TitledString('Weight:', 'Bold', VIEW_WIDTH));
            FW_EXTRABOLD:  Add(TitledString('Weight:', 'Extra bold', VIEW_WIDTH));
            FW_HEAVY:      Add(TitledString('Weight:', 'Heavy', VIEW_WIDTH));
        end;
        //I[o[nO
        Add(TitledString('Overhang:', tmOverhang, VIEW_WIDTH));
        //AXyNg
        Add(TitledString('Digitized Aspect X:', tmDigitizedAspectY, VIEW_WIDTH));
        //AXyNg
        Add(TitledString('Digitized Aspect Y:', tmDigitizedAspectX, VIEW_WIDTH));
        //ŏ̕
        Add(TitledString('First Character:', tmFirstChar, VIEW_WIDTH));
        //Ō̕
        Add(TitledString('Last Character:', tmLastChar, VIEW_WIDTH));
        //ftHg̕
        Add(TitledString('Default Char:', tmDefaultChar, VIEW_WIDTH));
        //؂蕶
        Add(TitledString('Break Char:', tmBreakChar, VIEW_WIDTH));
        //C^bNtO
        Add(TitledString('Italic Flag:', tmItalic > 0, VIEW_WIDTH));
        //tO
        Add(TitledString('Underlined Flag:', tmUnderlined > 0, VIEW_WIDTH));
        //łtO
        Add(TitledString('StruckOut Flag:', tmStruckOut > 0, VIEW_WIDTH));
        //sb`ƃt@~̃tO
        if ((tmPitchAndFamily and $0F) and TMPF_FIXED_PITCH)= TMPF_FIXED_PITCH then
            Add(TitledString('Pitch:', 'Fixed pitch', VIEW_WIDTH));
        if ((tmPitchAndFamily and $0F) and TMPF_VECTOR) = TMPF_VECTOR then
            Add(TitledString('Pitch:', 'Vector', VIEW_WIDTH));
        if ((tmPitchAndFamily and $0F) and TMPF_TRUETYPE) = TMPF_TRUETYPE then
            Add(TitledString('Pitch:', 'True Type', VIEW_WIDTH));
        if ((tmPitchAndFamily and $0F) and TMPF_DEVICE) = TMPF_DEVICE then
            Add(TitledString('Pitch:', 'Device', VIEW_WIDTH));
        if (tmPitchAndFamily and $0F) = 0 then
            Add(TitledString('Pitch:', 'Monospaced bitmap font', VIEW_WIDTH));

        case (tmPitchAndFamily and $F0) of
            FF_DECORATIVE: Add(TitledString('Family:', 'Decorative', VIEW_WIDTH));
            FF_DONTCARE:   Add(TitledString('Family:', 'Don''t care', VIEW_WIDTH));
            FF_MODERN:     Add(TitledString('Family:', 'Modern', VIEW_WIDTH));
            FF_ROMAN:      Add(TitledString('Family:', 'Roman', VIEW_WIDTH));
            FF_SCRIPT:     Add(TitledString('Family:', 'Script', VIEW_WIDTH));
            FF_SWISS:      Add(TitledString('Family:', 'Swiss', VIEW_WIDTH));
        end;
        //Zbg
        case tmCharSet of
            ANSI_CHARSET:        Add(TitledString('Character set:', 'ANSI', VIEW_WIDTH));
            DEFAULT_CHARSET:     Add(TitledString('Character set:', 'Default', VIEW_WIDTH));
            SYMBOL_CHARSET:      Add(TitledString('Character set:', 'Symbol', VIEW_WIDTH));
            SHIFTJIS_CHARSET:    Add(TitledString('Character set:', 'ShiftJis', VIEW_WIDTH));
            GB2312_CHARSET:      Add(TitledString('Character set:', 'GB2312', VIEW_WIDTH));
            HANGEUL_CHARSET:     Add(TitledString('Character set:', 'Hangeul', VIEW_WIDTH));
            CHINESEBIG5_CHARSET: Add(TitledString('Character set:', 'Chinese Big5', VIEW_WIDTH));
            OEM_CHARSET:         Add(TitledString('Character set:', 'OEM', VIEW_WIDTH));
        else
            Add('Windows 95 only character set');
        end;

    end;

end;
}

function	KanaToChar(AKana: string): Char;
var
	Index: integer;
begin
    Index := KanaTable.IndexOf(AKana);
    if Index <> -1 then
    	Result := Chr(Integer(KanaTable.Objects[Index]))
    else
    	Result := #0;
end;

function	KanaToChars(AKana: string): string;
var
	i: integer;
begin
    Result := '';
	i := 1;
    while i <= Length(AKana) do
    begin
		if IsLeadByte(AKana[i], 0) then
        begin
        	Result := Result + KanaToChar(AKana[i] + AKana[i + 1]);
            Inc(i);
            Inc(i);
        end
        else
        begin
        	Inc(i);
        end;
    end;
end;

procedure	SetKanaTable();
begin
	with KanaTable do
    begin
    	Clear;
		AddObject('', TObject('3'));
		AddObject('', TObject('e'));
		AddObject('', TObject('4'));
		AddObject('', TObject('5'));
		AddObject('', TObject('6'));

		AddObject('', TObject('t'));
		AddObject('', TObject('g'));
		AddObject('', TObject('h'));
		AddObject('', TObject(':'));
		AddObject('', TObject('b'));

		AddObject('', TObject('x'));
		AddObject('', TObject('d'));
		AddObject('', TObject('r'));
		AddObject('', TObject('p'));
		AddObject('', TObject('c'));

		AddObject('', TObject('q'));
		AddObject('', TObject('a'));
		AddObject('', TObject('z'));
		AddObject('', TObject('w'));
		AddObject('', TObject('s'));

		AddObject('', TObject('u'));
		AddObject('', TObject('i'));
		AddObject('', TObject('1'));
		AddObject('', TObject(','));
		AddObject('', TObject('k'));

		AddObject('', TObject('f'));
		AddObject('', TObject('v'));
		AddObject('', TObject('2'));
		AddObject('', TObject('^'));
		AddObject('', TObject('-'));

		AddObject('', TObject('j'));
		AddObject('', TObject('n'));
		AddObject('', TObject(']'));
		AddObject('', TObject('/'));
		AddObject('', TObject('m'));

		AddObject('', TObject('7'));
		AddObject('', TObject('8'));
		AddObject('', TObject('9'));

		AddObject('', TObject('o'));
		AddObject('', TObject('l'));
		AddObject('', TObject('.'));
		AddObject('', TObject(';'));
		AddObject('', TObject('N'));

		AddObject('', TObject('0'));
		AddObject('', TObject('}'));
		AddObject('', TObject('y'));

        //

		AddObject('', TObject('J'));
		AddObject('', TObject('G'));
		AddObject('', TObject('H'));
		AddObject('', TObject('*'));
		AddObject('', TObject('B'));

		AddObject('', TObject('X'));
		AddObject('', TObject('D'));
		AddObject('', TObject('R'));
		AddObject('', TObject('P'));
		AddObject('', TObject('C'));

		AddObject('', TObject('Q'));
		AddObject('', TObject('A'));
		AddObject('', TObject('M'));
		AddObject('', TObject('W'));
		AddObject('', TObject('S'));

		AddObject('', TObject('F'));
		AddObject('', TObject('V'));
		AddObject('', TObject('?'));
		AddObject('', TObject('`'));
		AddObject('', TObject('='));

		AddObject('', TObject('U'));
		AddObject('', TObject('I'));
		AddObject('', TObject('!'));
		AddObject('', TObject('<'));
		AddObject('', TObject('K'));

		AddObject('', TObject('#'));
		AddObject('', TObject('E'));
		AddObject('', TObject('$'));
		AddObject('', TObject('%'));
		AddObject('', TObject('&'));

		AddObject('', TObject('('));
		AddObject('', TObject(')'));
		AddObject('', TObject('Z'));
		AddObject('|', TObject('['));
        
    end;

end;

initialization

	KanaTable := TStringList.Create;
    SetKanaTable();

finalization

	KanaTable.Free;

end.                                                            
